package es.kauron.jstudy.controller;

import es.kauron.jstudy.Main;
import es.kauron.jstudy.model.TestItem;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Parent;
import javafx.scene.control.*;
import javafx.scene.layout.BorderPane;
import javafx.stage.FileChooser;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.*;

public class Controller implements Initializable {
    @FXML
    private TabPane tabPane;

    @FXML
    private BorderPane root;
    @FXML
    private ToolBar toolbar;
    @FXML
    private MenuItem saveMenu;

    private BooleanProperty table = new SimpleBooleanProperty(false);
    private Map<Tab, TableController> tabMap = new HashMap<>();

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        tabPane.getSelectionModel().selectedItemProperty().addListener((change, o, n) -> {
            if (tabMap.get(n) != null) {
                table.set(true);
            } else table.set(false);
        });
        saveMenu.disableProperty().bind(table.not());
    }

    @FXML
    private void onSettingsAction(ActionEvent event) {
        try {
            Parent root = FXMLLoader.load(Main.class.getResource("view/settings.fxml"));
            tabPane.getTabs().add(new Tab("Settings", root));
            tabPane.getSelectionModel().selectLast();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @FXML
    private void onNewAction(ActionEvent event) {
        TextInputDialog dialog = new TextInputDialog("Name");
        dialog.showAndWait();
        tabPane.getTabs().add(createTableTab(dialog.getResult(), new ArrayList<>()));
        tabPane.getSelectionModel().selectLast();
    }

    @FXML
    private void onLoadAction(ActionEvent event) {
        FileChooser chooser = new FileChooser();
        chooser.getExtensionFilters().add(new FileChooser.ExtensionFilter("JStudy file", "*.jsdb"));
        File file = chooser.showOpenDialog(root.getScene().getWindow());
        if (file == null) return;
        List<TestItem> aux = TestItem.loadFrom(file, TestItem.COLONS);
        if (aux != null) {
            tabPane.getTabs().add(createTableTab(file.getName().substring(0, file.getName().lastIndexOf('.')), aux));
            tabPane.getSelectionModel().selectLast();
        }
    }

    @FXML
    private void onSaveAction(ActionEvent event) {
        FileChooser chooser = new FileChooser();
        chooser.getExtensionFilters().add(new FileChooser.ExtensionFilter("JStudy file", "*.jsdb"));
        File file = chooser.showSaveDialog(root.getScene().getWindow());
        if (file == null) return;
        TestItem.saveTo(file, tabMap.get(tabPane.getSelectionModel().getSelectedItem()).getData());
    }

    @FXML
    private void onImportAction(ActionEvent event) {
        FileChooser chooser = new FileChooser();
        chooser.getExtensionFilters().add(new FileChooser.ExtensionFilter("Data files (.csv)", "*.csv"));
        File file = chooser.showOpenDialog(root.getScene().getWindow());
        if (file == null) return;
        List<TestItem> aux = TestItem.loadFrom(file, TestItem.SEMICOLON);
        if (aux != null) {
            tabPane.getTabs().add(createTableTab(file.getName().substring(0, file.getName().lastIndexOf('.')), aux));
            tabPane.getSelectionModel().selectLast();
        }
    }

    private Tab createTableTab(String name, List<TestItem> list) {
        try {
            Tab tab = new Tab(name);

            FXMLLoader loader = new FXMLLoader(Main.class.getResource("view/table.fxml"));
            Parent tableRoot = loader.load();

            ((TableController) loader.getController()).setData(list, this);
            tabMap.put(tab, loader.getController());

            tab.setContent(tableRoot);
            return tab;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    void newTest(List<TestItem> list) {
        try {
            FXMLLoader loader = new FXMLLoader(Main.class.getResource("view/test.fxml"));
            Parent root = loader.load();

            ((TestController) loader.getController()).setList(new ArrayList<>(list));

            tabPane.getTabs().add(new Tab("Test: " + tabPane.getSelectionModel().getSelectedItem().getText(), root));
            tabPane.getSelectionModel().selectLast();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
