package es.kauron.jstudy.controller;

import com.jfoenix.controls.JFXTextField;
import es.kauron.jstudy.model.AppConfig;
import es.kauron.jstudy.model.TestItem;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;
import javafx.scene.control.ProgressBar;
import javafx.scene.layout.HBox;

import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

public class TestController implements Initializable {
    @FXML
    private Label question, prevQuestion, prevAnswer, correctAnswer, progressLabel, correctLabel;
    @FXML
    private JFXTextField answer;
    @FXML
    private ProgressBar progress;
    @FXML
    private HBox feedback;

    private List<TestItem> list;
    private int total, current;

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        feedback.setVisible(AppConfig.showFeedback);
    }

    void setList(List<TestItem> list) {
        this.list = list;
        total = list.size();
        progress.setProgress(0);
        progressLabel.setText("0/" + total);
        onSkipAction(null);
    }

    @FXML
    private void onNextAction(ActionEvent event) {
        prevQuestion.setText(list.get(current).getQuestion());
        prevAnswer.setText(answer.getText());
        correctAnswer.setText(list.get(current).getAnswer());

        boolean right = list.get(current).getAnswer().equals(answer.getText());
        correctAnswer.setVisible(!right);
        correctLabel.setVisible(!right);

        // Remove the question from the pool if the question was correctly answered or there is no repetition
        if (right || !AppConfig.repeatWrong) {
            list.remove(current);
            progress.setProgress(progress.getProgress() + 1.0/total);
            progressLabel.setText((int) (progress.getProgress() * total) + "/" + total);
            if (list.size() == 0) {
                answer.setText("");
                answer.setDisable(true);
                question.setText("That's it!");
                return;
            }
        }
        onSkipAction(event);
    }

    @FXML
    private void onSkipAction(ActionEvent event) {
        feedback.setVisible(AppConfig.showFeedback);
        answer.setText("");
        current = (int) (Math.random() * list.size());
        question.setText(list.get(current).getQuestion());
        answer.requestFocus();
    }
}
